@@ -4,14 +4,16 @@
 -- Module      :  Network
 -- Copyright   :  (c) The University of Glasgow 2001
 -- License     :  BSD-style (see the file libraries/network/LICENSE)
--- 
+--
 -- Maintainer  :  libraries@haskell.org
 -- Stability   :  provisional
 -- Portability :  portable
 --
--- The "Network" interface is a \"higher-level\" interface to
--- networking facilities, and it is recommended unless you need the
--- lower-level interface in "Network.Socket".
+-- This module is kept for backwards-compatibility. New users are
+-- encouraged to use "Network.Socket" instead.
+--
+-- "Network" was intended as a \"higher-level\" interface to networking
+-- facilities, and only supports TCP.
 --
 -----------------------------------------------------------------------------
 
@@ -32,7 +34,7 @@ module Network
 
     -- * Initialisation
     , withSocketsDo
-    
+
     -- * Server-side connections
     , listenOn
     , accept
@@ -63,7 +65,8 @@ module Network
 import Control.Monad (liftM)
 import Data.Maybe (fromJust)
 import Network.BSD
-import Network.Socket hiding (accept, socketPort, recvFrom, sendTo, PortNumber)
+import Network.Socket hiding (accept, socketPort, recvFrom,
+                              sendTo, PortNumber, sClose)
 import qualified Network.Socket as Socket (accept)
 import System.IO
 import Prelude
@@ -77,7 +80,7 @@ import qualified Control.Exception as Exception
 -- raised. Alternatively an empty string may be given to @connectTo@
 -- signalling that the current hostname applies.
 
-data PortID = 
+data PortID =
           Service String                -- Service Name eg "ftp"
         | PortNumber PortNumber         -- User defined Port Number
 #if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
@@ -88,7 +91,7 @@ data PortID =
 -- | Calling 'connectTo' creates a client side socket which is
 -- connected to the given host and port.  The Protocol and socket type is
 -- derived from the given port identifier.  If a port number is given
--- then the result is always an internet family 'Stream' socket. 
+-- then the result is always an internet family 'Stream' socket.
 
 connectTo :: HostName           -- Hostname
           -> PortID             -- Port Identifier
@@ -267,14 +270,14 @@ listen' serv = do
 accept :: Socket                -- ^ Listening Socket
        -> IO (Handle,
               HostName,
-              PortNumber)       -- ^ Triple of: read\/write 'Handle' for 
+              PortNumber)       -- ^ Triple of: read\/write 'Handle' for
                                 -- communicating with the client,
                                 -- the 'HostName' of the peer socket, and
                                 -- the 'PortNumber' of the remote connection.
 accept sock@(MkSocket _ AF_INET _ _ _) = do
  ~(sock', (SockAddrInet port haddr)) <- Socket.accept sock
  peer <- catchIO
-          (do   
+          (do
              (HostEntry peer _ _ _) <- getHostByAddr AF_INET haddr
              return peer
           )
@@ -308,6 +311,13 @@ accept sock@(MkSocket _ AF_UNIX _ _ _) = do
 accept (MkSocket _ family _ _ _) =
   error $ "Sorry, address family " ++ (show family) ++ " is not supported!"
 
+
+-- | Close the socket. All future operations on the socket object will fail.
+--   The remote end will receive no more data (after queued data is flushed).
+sClose :: Socket -> IO ()
+sClose = close -- Explicit redefinition because Network.sClose is deperecated,
+               -- hence the re-export would also be marked as such.
+
 -- -----------------------------------------------------------------------------
 -- sendTo/recvFrom
 
@@ -361,7 +371,7 @@ recvFrom host port = do
  ip  <- getHostByName host
  let ipHs = hostAddresses ip
  s   <- listenOn port
- let 
+ let
   waiting = do
      ~(s', SockAddrInet _ haddr)  <-  Socket.accept s
      he <- getHostByAddr AF_INET haddr
@@ -400,25 +410,14 @@ socketPort s = do
 -- ---------------------------------------------------------------------------
 -- Utils
 
--- Like bracket, but only performs the final action if there was an 
+-- Like bracket, but only performs the final action if there was an
 -- exception raised by the middle bit.
 bracketOnError
         :: IO a         -- ^ computation to run first (\"acquire resource\")
         -> (a -> IO b)  -- ^ computation to run last (\"release resource\")
         -> (a -> IO c)  -- ^ computation to run in-between
         -> IO c         -- returns the value from the in-between computation
-#if __GLASGOW_HASKELL__ && __GLASGOW_HASKELL__ < 606
-bracketOnError before after thing =
-  Exception.block (do
-    a <- before
-    r <- Exception.catch
-           (Exception.unblock (thing a))
-           (\e -> do { after a; Exception.throw e })
-    return r
- )
-#else
 bracketOnError = Exception.bracketOnError
-#endif
 
 -----------------------------------------------------------------------------
 -- Extra documentation
@@ -462,12 +461,26 @@ catchIO = Exception.catch
 catchIO = Exception.catchJust Exception.ioErrors
 #endif
 
+-- Version of try implemented in terms of the locally defined catchIO
+tryIO :: IO a -> IO (Either Exception.IOException a)
+tryIO m = catchIO (liftM Right m) (return . Left)
+
 -- Returns the first action from a list which does not throw an exception.
 -- If all the actions throw exceptions (and the list of actions is not empty),
 -- the last exception is thrown.
+-- The operations are run outside of the catchIO cleanup handler because
+-- catchIO masks asynchronous exceptions in the cleanup handler.
+-- In the case of complete failure, the last exception is actually thrown.
 firstSuccessful :: [IO a] -> IO a
-firstSuccessful [] = error "firstSuccessful: empty list"
-firstSuccessful (p:ps) = catchIO p $ \e ->
-    case ps of
-        [] -> Exception.throwIO e
-        _  -> firstSuccessful ps
+firstSuccessful = go Nothing
+  where
+  -- Attempt the next operation, remember exception on failure
+  go _ (p:ps) =
+    do r <- tryIO p
+       case r of
+         Right x -> return x
+         Left  e -> go (Just e) ps
+
+  -- All operations failed, throw error if one exists
+  go Nothing  [] = error "firstSuccessful: empty list"
+  go (Just e) [] = Exception.throwIO e
