@@ -1,14 +1,31 @@
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 
 module Main where
 
 import Control.Concurrent (ThreadId, forkIO, myThreadId)
 import Control.Concurrent.MVar (newEmptyMVar, putMVar, takeMVar, readMVar)
 import qualified Control.Exception as E
+import Control.Monad (liftM, when)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as C
+import Data.Maybe (fromJust)
 import Network.Socket hiding (recv, recvFrom, send, sendTo)
 import Network.Socket.ByteString
+
+--- To tests for AF_CAN on Linux, you need to bring up a virtual (or real can
+--- interface.). Run as root:
+--- # modprobe can
+--- # modprobe can_raw
+--- # modprobe vcan
+--- # sudo ip link add dev vcan0 type vcan
+--- # ip link show vcan0
+--- 3: can0: <NOARP,UP,LOWER_UP> mtu 16 qdisc noqueue state UNKNOWN link/can
+--- Define HAVE_LINUX_CAN to run CAN tests as well.
+--- #define HAVE_LINUX_CAN 1
+-- #include "../include/HsNetworkConfig.h"
+#if defined(HAVE_LINUX_CAN_H)
+import Network.BSD (ifNameToIndex)
+#endif
 import Test.Framework (Test, defaultMain, testGroup)
 import Test.Framework.Providers.HUnit (testCase)
 import Test.HUnit (Assertion, (@=?))
@@ -112,6 +129,104 @@ testOverFlowRecvFrom = tcpTest client server
 
     client sock = send sock testMsg
 
+testUserTimeout :: Assertion
+testUserTimeout = do
+    when (isSupportedSocketOption UserTimeout) $ do
+      sock <- socket AF_INET Stream defaultProtocol
+      setSocketOption sock UserTimeout 1000
+      getSocketOption sock UserTimeout >>= (@=?) 1000
+      setSocketOption sock UserTimeout 2000
+      getSocketOption sock UserTimeout >>= (@=?) 2000
+      sClose sock
+
+{-
+testGetPeerCred:: Assertion
+testGetPeerCred =
+    test clientSetup clientAct serverSetup server
+  where
+    clientSetup = do
+        sock <- socket AF_UNIX Stream defaultProtocol
+        connect sock $ SockAddrUnix addr 
+        return sock
+
+    serverSetup = do
+        sock <- socket AF_UNIX Stream defaultProtocol
+        bindSocket sock $ SockAddrUnix addr 
+        listen sock 1
+        return sock
+
+    server sock = do
+        (clientSock, _) <- accept sock
+        serverAct clientSock
+        sClose clientSock
+
+    addr = "/tmp/testAddr1"
+    clientAct sock = withSocketsDo $ do  
+                     sendAll sock testMsg
+                     (pid,uid,gid) <- getPeerCred sock
+                     putStrLn $ unwords ["pid=",show pid,"uid=",show uid, "gid=", show gid]
+    serverAct sock = withSocketsDo $ do
+                     msg <- recv sock 1024
+                     putStrLn $ C.unpack msg
+
+
+testGetPeerEid :: Assertion
+testGetPeerEid =  
+    test clientSetup clientAct serverSetup server
+  where
+    clientSetup = do
+        sock <- socket AF_UNIX Stream defaultProtocol
+        connect sock $ SockAddrUnix addr 
+        return sock
+
+    serverSetup = do
+        sock <- socket AF_UNIX Stream defaultProtocol
+        bindSocket sock $ SockAddrUnix addr 
+        listen sock 1
+        return sock
+
+    server sock = do
+        (clientSock, _) <- accept sock
+        serverAct clientSock
+        sClose clientSock
+
+    addr = "/tmp/testAddr2"
+    clientAct sock = withSocketsDo $ do  
+                     sendAll sock testMsg
+                     (uid,gid) <- getPeerEid sock
+                     putStrLn $ unwords ["uid=",show uid, "gid=", show gid]
+    serverAct sock = withSocketsDo $ do
+                     msg <- recv sock 1024
+                     putStrLn $ C.unpack msg
+-}
+
+#if defined(HAVE_LINUX_CAN_H)
+canTestMsg = S.pack [ 0,0,0,0 -- can ID = 0
+                    , 4,0,0,0 -- data length counter = 2 (bytes)
+                    , 0x80,123,321,55 -- SYNC with some random extra bytes
+                    , 0, 0, 0, 0 -- padding
+                    ]
+
+testCanSend :: Assertion
+testCanSend = canTest "vcan0" client server
+  where
+    server sock = recv sock 1024 >>= (@=?) canTestMsg
+    client sock = send sock canTestMsg
+
+canTest :: String -> (Socket -> IO a) -> (Socket -> IO b) -> IO ()
+canTest ifname clientAct serverAct = do
+    ifIndex <- liftM fromJust $ ifNameToIndex ifname
+    test (clientSetup ifIndex) clientAct (serverSetup ifIndex) serverAct
+  where
+    clientSetup ifIndex = do
+      sock <- socket AF_CAN Raw 1 -- protocol 1 = raw CAN
+      -- bind the socket to the interface
+      bind sock (SockAddrCan $ fromIntegral $ ifIndex)
+      return sock
+    
+    serverSetup = clientSetup
+#endif
+
 ------------------------------------------------------------------------
 -- Other
 
@@ -132,6 +247,12 @@ basicTests = testGroup "Basic socket operations"
     , testCase "testOverFlowRecv" testOverFlowRecv
     , testCase "testRecvFrom" testRecvFrom
     , testCase "testOverFlowRecvFrom" testOverFlowRecvFrom
+    , testCase "testUserTimeout" testUserTimeout
+--    , testCase "testGetPeerCred" testGetPeerCred
+--    , testCase "testGetPeerEid" testGetPeerEid
+#if defined(HAVE_LINUX_CAN_H)
+    , testCase "testCanSend" testCanSend  
+#endif
     ]
 
 tests :: [Test]
